import Flutter
import UIKit
import CoreTelephony

public class SwiftNetworkingPermissionsPlugin: NSObject, FlutterPlugin {
    /// 初始化马上获取可能得到状态为kCTCellularDataUnknow，所以最好延迟获取，cellularData对象不要每次创建新的
    let cellularData = CTCellularData()
    var eventSink: FlutterEventSink?
    
    
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "networking_permissions_method", binaryMessenger: registrar.messenger())
        let streamChannel = FlutterEventChannel(name: "networking_permissions_event", binaryMessenger: registrar.messenger())
        let instance = SwiftNetworkingPermissionsPlugin()
        registrar.addApplicationDelegate(instance)
        streamChannel.setStreamHandler(instance)
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    
    public func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [AnyHashable : Any] = [:]) -> Bool {
        getNetworkingPermissionsStatus()
        return true
    }
    

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if (call.method == "getNetworkingPermissions") {
            let status = getNetworkingPermissions()
            result(status)
        } else {
            result("unknown method")
        }
        
    }
    
    func getNetworkingPermissions() -> String {
        var status = "restrictedStateUnknown"
        if #available(iOS 9.0, *) {
            let state = cellularData.restrictedState
            print("RestrictedState:\(state)")
            status = statusFrom(restrictedState: state)
        } else {
            status = "notRestricted"
        }
        print("Permission:\(status)")
        return status
    }
    
    
    func getNetworkingPermissionsStatus() {
        cellularData.cellularDataRestrictionDidUpdateNotifier = { [weak self] (state) in
            guard let self = self else { return }
            if let sink = self.eventSink {
                print("notifier:\(self.statusFrom(restrictedState: state))")
                sink(self.statusFrom(restrictedState: state))
            }
        }
    }
    
    private func statusFrom(restrictedState: CTCellularDataRestrictedState) -> String {
        switch restrictedState {
        case .restricted:
            return "restricted"
        case .notRestricted:
            return "notRestricted"
        case .restrictedStateUnknown:
            return "restrictedStateUnknown"
        @unknown default:
            return "restrictedStateUnknown"
        }
    }
    
    
    
    
}

extension SwiftNetworkingPermissionsPlugin:FlutterStreamHandler {
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        eventSink = events
        return nil
    }
    
    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        eventSink = nil
        return nil
    }
}
