import 'dart:async';

import 'package:flutter/services.dart';

enum CTCellularDataRestrictedState {
  restrictedStateUnknown,
  restricted,
  notRestricted,
}

class NetworkingPermissions {
  factory NetworkingPermissions() {
    _singleton ??= NetworkingPermissions._();
    return _singleton!;
  }

  NetworkingPermissions._();

  static NetworkingPermissions? _singleton;

  final MethodChannel _methodChannel = const MethodChannel('networking_permissions_method');

  final EventChannel _eventChannel = const EventChannel('networking_permissions_event');

  Future<CTCellularDataRestrictedState> get getNetworkingPermissions async {
    final String? state = await _methodChannel.invokeMethod('getNetworkingPermissions');
    return parseResultToState(state);
  }

  Stream<CTCellularDataRestrictedState> getNetworkingPermissionsStatus() {
    return _eventChannel.receiveBroadcastStream().map((dynamic result) => result.toString()).map(parseResultToState);
  }
}

CTCellularDataRestrictedState parseResultToState(String? state) {
  switch (state) {
    case 'restricted':
      return CTCellularDataRestrictedState.restricted;
    case 'notRestricted':
      return CTCellularDataRestrictedState.notRestricted;
    case 'restrictedStateUnknown':
      return CTCellularDataRestrictedState.restrictedStateUnknown;
    default:
      return CTCellularDataRestrictedState.restrictedStateUnknown;
  }
}

String parseResultToString(CTCellularDataRestrictedState? state) {
  if (state == null) {
    return 'restrictedStateUnknown';
  }
  switch (state) {
    case CTCellularDataRestrictedState.restrictedStateUnknown:
      return 'restrictedStateUnknown';
    case CTCellularDataRestrictedState.restricted:
      return 'restricted';
    case CTCellularDataRestrictedState.notRestricted:
      return 'notrestricted';
  }
}
